/*
Deployment script for Sitecore_analytics_81u3

This code was generated by a tool.
Changes to this file may cause incorrect behavior and will be lost if
the code is regenerated.
*/

GO
SET ANSI_NULLS, ANSI_PADDING, ANSI_WARNINGS, ARITHABORT, CONCAT_NULL_YIELDS_NULL, QUOTED_IDENTIFIER ON;

SET NUMERIC_ROUNDABORT OFF;

GO


GO
PRINT N'Altering [dbo].[DimensionKeys]...';


GO

if exists(select top 1 * from sys.columns 
where [object_id] = object_id('DimensionKeys') and name = 'DimensionKey' )
ALTER TABLE [dbo].[DimensionKeys] ALTER COLUMN [DimensionKey] NVARCHAR (MAX) NOT NULL;


GO

IF( EXISTS( SELECT 1 FROM [information_schema].[tables] WHERE ([table_schema] = 'dbo') AND ([table_type] = 'BASE TABLE') AND ([table_name] = 'FunnelDefinitions') ) )
  DROP TABLE [dbo].[FunnelDefinitions];
ELSE
  PRINT N'Creating [dbo].[FunnelDefinitions]...';

GO
CREATE TABLE [dbo].[FunnelDefinitions] (
    [Id]       UNIQUEIDENTIFIER NOT NULL,
    [Version]  SMALLINT         NOT NULL,
    [Language] NVARCHAR (10)    NOT NULL,
    [IsActive] BIT              NULL,
    [Data]     VARBINARY (MAX)  NULL,
    CONSTRAINT [PK_FunnelDefinitions] PRIMARY KEY CLUSTERED ([Id] ASC, [Version] ASC, [Language] ASC)
);


GO

IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Trees]') AND name = N'IX_Trees_Visits')
BEGIN
  PRINT N'Creating [dbo].[Trees].[IX_Trees_Visits]...';
  CREATE NONCLUSTERED INDEX [IX_Trees_Visits]
    ON [dbo].[Trees]([Visits] ASC);
END;

GO
PRINT N'Refreshing [dbo].[ReportDataView]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[ReportDataView]';


GO
PRINT N'Altering [dbo].[ReduceSegmentMetrics]...';


GO

ALTER PROCEDURE [dbo].[ReduceSegmentMetrics]
  @SiteNameId int,
  @SegmentId uniqueidentifier,
  @OtherSegmentRecordId bigint,
  @StartDate smalldatetime,
  @OtherDimensionKeyId bigint,
  @OtherDimensionKey nvarchar(450),
  @VisitThreshold int = 10,
  @ValueThreshold int = -1,
  @KeepCountThreshold int = 1000,
  @DryRun bit = 1,
  @Debug bit = 0,
  @UnclassifiedDeviceKey nvarchar(450)
AS
BEGIN
    SET NOCOUNT ON;

    DECLARE @EndDate smalldatetime
    SET @EndDate = DATEADD(day, 1, @StartDate)

    -- Copy segment data for given date range
    -- Compute order value
    CREATE TABLE [dbo].[#tmp](
        [PredicateOrder] [bigint] NULL,
        [SegmentId] [uniqueidentifier] NOT NULL,
        [Date] [smalldatetime] NOT NULL,
        [SiteNameId] [int] NOT NULL,
        [DimensionKeyId] [bigint] NOT NULL,
        [SegmentRecordId] [bigint] NOT NULL,
        [ContactTransitionType] [tinyint] NOT NULL,
        [Visits] [int] NOT NULL,
        [Value] [int] NOT NULL,
        [Bounces] [int] NOT NULL,
        [Conversions] [int] NOT NULL,
        [TimeOnSite] [int] NOT NULL,
        [Pageviews] [int] NOT NULL,
        [Count] [int] NOT NULL
    ) ON [PRIMARY]

    INSERT #tmp
    SELECT
        ROW_NUMBER() OVER(ORDER BY sm.Visits DESC, ABS(sm.Value) DESC) AS 'PredicateOrder',
        sr.[SegmentId],
        sr.[Date],
        sr.[SiteNameId],
        sr.[DimensionKeyId],
        sm.[SegmentRecordId],
        sm.[ContactTransitionType],
        sm.[Visits],
        sm.[Value],
        sm.[Bounces],
        sm.[Conversions],
        sm.[TimeOnSite],
        sm.[Pageviews],
        sm.[Count]
    FROM SegmentRecords sr
        INNER JOIN Fact_SegmentMetrics sm ON sr.SegmentRecordId = sm.SegmentRecordId
        INNER JOIN DimensionKeys dk ON dk.DimensionKeyId = sr.DimensionKeyId
    WHERE sr.SegmentId = @SegmentId AND sr.[Date] >= @StartDate AND 
        sr.[Date] < @EndDate AND
        sr.[SiteNameId] = @SiteNameId AND
        dk.DimensionKey Not Like @UnclassifiedDeviceKey + '%'

    -- Compute how many records to keep
    DECLARE @KeepCount int
    SELECT @KeepCount = MAX(PredicateOrder)
    FROM #tmp

    IF(@KeepCount > @KeepCountThreshold) 
        SET @KeepCount = @KeepCountThreshold

    ELSE
    BEGIN
        SELECT @KeepCount = MAX(PredicateOrder)
        FROM #tmp 
        WHERE Visits > @VisitThreshold AND ABS(Value) > @ValueThreshold

        IF(@KeepCount IS NULL)
            SET @KeepCount = 0
    END 

    -- Populate SegmentRecordsReduced from temp table
    -- Use top KeepCount records (highest predicate value)
    MERGE
    [SegmentRecordsReduced] AS [target]
    USING
    (
        SELECT DISTINCT [SegmentRecordId], [SegmentId], [Date], [SiteNameId], [DimensionKeyId]
        FROM #tmp
        WHERE PredicateOrder <= @KeepCount
    )
    AS [source] ([SegmentRecordId], [SegmentId], [Date], [SiteNameId], [DimensionKeyId])
    ON
    (
        [target].[SegmentRecordId] = [source].[SegmentRecordId]
    )
    WHEN MATCHED THEN
        UPDATE
        SET
            [target].[SegmentId] = [source].[SegmentId],
            [target].[Date] = [source].[Date],
            [target].[SiteNameId] = [source].[SiteNameId],
            [target].[DimensionKeyId] = [source].[DimensionKeyId]
    WHEN NOT MATCHED THEN
    INSERT ([SegmentRecordId], [SegmentId], [Date], [SiteNameId], [DimensionKeyId])
    VALUES
    (
        [source].[SegmentRecordId],
        [source].[SegmentId],
        [source].[Date],
        [source].[SiteNameId],
        [source].[DimensionKeyId]
    );


    -- Populate Fact_SegmentMetricsReduced from temp table
    -- Use top KeepCount records (highest predicate value)
    MERGE
    [Fact_SegmentMetricsReduced] AS [target]
    USING
    (
        SELECT [SegmentRecordId], [ContactTransitionType], [Visits], [Value], [Bounces], [Conversions], [TimeOnSite], [Pageviews], [Count]
        FROM #tmp
        WHERE PredicateOrder <= @KeepCount
    )
    AS [source] ([SegmentRecordId], [ContactTransitionType], [Visits], [Value], [Bounces], [Conversions], [TimeOnSite], [Pageviews], [Count])
    ON 
    (
        [target].[SegmentRecordId] = [source].[SegmentRecordId] AND [target].[ContactTransitionType] = [source].[ContactTransitionType]
    )
    WHEN MATCHED THEN
        UPDATE
        SET
            [target].[Visits] = [source].[Visits] + [target].[Visits],
            [target].[Value] = [source].[Value] + [target].[Value],
            [target].[Bounces] = [source].[Bounces] + [target].[Bounces],
            [target].[Conversions] = [source].[Conversions] + [target].[Conversions],
            [target].[TimeOnSite] = [source].[TimeOnSite] + [target].[TimeOnSite],
            [target].[Pageviews] = [source].[Pageviews] + [target].[Pageviews],
            [target].[Count] = [source].[Count] + [target].[Count]
    WHEN NOT MATCHED THEN
    INSERT ([SegmentRecordId], [ContactTransitionType], [Visits], [Value], [Bounces], [Conversions], [TimeOnSite], [Pageviews], [Count])
    VALUES
    (
        [source].[SegmentRecordId],
        [source].[ContactTransitionType],
        [source].[Visits],
        [source].[Value],
        [source].[Bounces],
        [source].[Conversions],
        [source].[TimeOnSite],
        [source].[Pageviews],
        [source].[Count]
    );


    -- Populate DimensionKeys with record for "other"
    MERGE
        [DimensionKeys] AS [target]
    USING
    (
       VALUES (@OtherDimensionKeyId, @OtherDimensionKey)
    )
    AS [source] ([DimensionKeyId], [DimensionKey])
    ON
    (
        [target].[DimensionKeyId] = [source].[DimensionKeyId]
    )
    WHEN NOT MATCHED THEN
    INSERT ([DimensionKeyId], [DimensionKey])
    VALUES
    (
        [source].[DimensionKeyId],
        [source].[DimensionKey]
    );

    -- Populate SegmentRecordsReduced with one record for "other" if some records were reduced
    IF (EXISTS (SELECT * FROM  #tmp WHERE PredicateOrder > @KeepCount))
    MERGE
    [SegmentRecordsReduced] AS [target]
    USING
    (
       VALUES (@OtherSegmentRecordId, @SegmentId, @StartDate, @SiteNameId, @OtherDimensionKeyId)
    )
    AS [source] ([SegmentRecordId], [SegmentId], [Date], [SiteNameId], [DimensionKeyId])
    ON
    (
        [target].[SegmentRecordId] = [source].[SegmentRecordId]
    )
    WHEN MATCHED THEN
        UPDATE
        SET
            [target].[SegmentId] = [source].[SegmentId],
            [target].[Date] = [source].[Date],
            [target].[SiteNameId] = [source].[SiteNameId],
            [target].[DimensionKeyId] = [source].[DimensionKeyId]
    WHEN NOT MATCHED THEN
    INSERT ([SegmentRecordId], [SegmentId], [Date], [SiteNameId], [DimensionKeyId])
    VALUES
    (
        [source].[SegmentRecordId],
        [source].[SegmentId],
        [source].[Date],
        [source].[SiteNameId],
        [source].[DimensionKeyId]
    );

    -- Populate Fact_SegmentMetricsReduced from temp table with "other" records.
    -- Other is a summary of bottom records (lowest predicate value; above KeepCount)
    MERGE
    [Fact_SegmentMetricsReduced] AS [target]
    USING
    (
        SELECT
        @OtherSegmentRecordId AS [SegmentRecordId],
        [ContactTransitionType],
        SUM([Visits]) AS [Visits],
        SUM([Value]) AS [Value],
        SUM([Bounces]) AS [Bounces],
        SUM([Conversions]) AS [Conversions],
        SUM([TimeOnSite]) AS [TimeOnSite],
        SUM([Pageviews]) AS [Pageviews],
        SUM([Count]) AS [Count]
        FROM #tmp
        WHERE PredicateOrder > @KeepCount
        GROUP BY ContactTransitionType
    )
    AS [source] ([SegmentRecordId], [ContactTransitionType], [Visits], [Value], [Bounces], [Conversions], [TimeOnSite], [Pageviews], [Count])
    ON 
    (
        [target].[SegmentRecordId] = [source].[SegmentRecordId] AND 
        [target].[ContactTransitionType] = [source].[ContactTransitionType]
    )
    WHEN MATCHED THEN
        UPDATE
        SET
            [target].[Visits] = [source].[Visits] + [target].[Visits],
            [target].[Value] = [source].[Value] + [target].[Value],
            [target].[Bounces] = [source].[Bounces] + [target].[Bounces],
            [target].[Conversions] = [source].[Conversions] + [target].[Conversions],
            [target].[TimeOnSite] = [source].[TimeOnSite] + [target].[TimeOnSite],
            [target].[Pageviews] = [source].[Pageviews] + [target].[Pageviews],
            [target].[Count] = [source].[Count] + [target].[Count]
    WHEN NOT MATCHED THEN
    INSERT ([SegmentRecordId], [ContactTransitionType], [Visits], [Value], [Bounces], [Conversions], [TimeOnSite], [Pageviews], [Count])
    VALUES
    (
        [source].[SegmentRecordId],
        [source].[ContactTransitionType],
        [source].[Visits],
        [source].[Value],
        [source].[Bounces],
        [source].[Conversions],
        [source].[TimeOnSite],
        [source].[Pageviews],
        [source].[Count]
    );

    --pruning trace
    IF @Debug = 1
    BEGIN
        SELECT * FROM Fact_SegmentMetrics
        INNER JOIN SegmentRecords
        ON Fact_SegmentMetrics.SegmentRecordId=SegmentRecords.SegmentRecordId
        WHERE Fact_SegmentMetrics.SegmentRecordId IN (SELECT SegmentRecordId FROM #tmp)
    END

    -- Delete processed data from Fact_SegmentMetrics and SegmentRecords

    IF @DryRun = 0
    BEGIN
        PRINT N'Deleting source records.';
        DELETE Fact_SegmentMetrics WHERE SegmentRecordId IN (SELECT SegmentRecordId FROM #tmp)
        DELETE SegmentRecords WHERE SegmentRecordId IN (SELECT SegmentRecordId FROM #tmp)
    END
    ELSE
    BEGIN
        PRINT N'Executed in dry run mode.';
    END

    DROP TABLE #tmp
END


GO

IF OBJECT_ID ( 'Delete_FunnelDefinition', 'P' ) IS NOT NULL 
  DROP PROCEDURE [dbo].[Delete_FunnelDefinition];
ELSE
  PRINT N'Creating [dbo].[Delete_FunnelDefinition]...';

GO

CREATE PROCEDURE [dbo].[Delete_FunnelDefinition]
    @Id uniqueidentifier,
    @Language nvarchar(10) = null
AS
BEGIN
    IF @Language IS NULL 
    BEGIN
      Delete From [FunnelDefinitions]
      WHERE [Id] = @Id
    END
    ELSE 
        BEGIN
      Delete From [FunnelDefinitions]
      WHERE [Id] = @Id AND
      [Language] = @Language
        END
END
GO

IF OBJECT_ID ( 'Upsert_FunnelDefinition', 'P' ) IS NOT NULL 
  DROP PROCEDURE [dbo].[Upsert_FunnelDefinition];
ELSE
  PRINT N'Creating [dbo].[Upsert_FunnelDefinition]...';

GO
Create PROCEDURE [dbo].[Upsert_FunnelDefinition]
    @Id uniqueidentifier,
    @Version int,
    @Language nvarchar(50),
    @IsActive bit,
    @Data varbinary(max)
AS
BEGIN
    SET NOCOUNT ON;

    IF EXISTS
    (
        SELECT	1
        FROM	[FunnelDefinitions]
        WHERE	[Id] = @Id AND 
                [Version] = @Version AND 
                [Language] = @Language
    )
    BEGIN
        UPDATE	[FunnelDefinitions]
        SET		[Data] = @Data, [IsActive] = @IsActive
        WHERE	[Id] = @Id AND
                [Version] = @Version AND 
                [Language] = @Language
    END
    ELSE
    BEGIN
        INSERT INTO	[FunnelDefinitions] ([Id], [Version], [Language], [Data], [IsActive])
        VALUES		(@Id, @Version, @Language, @Data, @IsActive);
    END
END
GO

IF OBJECT_ID ( 'Ensure_DimensionKeys', 'P' ) IS NOT NULL 
	BEGIN
		PRINT N'Dropping [dbo].[Ensure_DimensionKeys]...';	
		DROP PROCEDURE [dbo].[Ensure_DimensionKeys]
	END

GO

IF OBJECT_ID ( 'Ensure_DimensionKeys_Tvp', 'P' ) IS NOT NULL 
	BEGIN
		PRINT N'Dropping [dbo].[Ensure_DimensionKeys_Tvp]...';	
		DROP PROCEDURE [dbo].[Ensure_DimensionKeys_Tvp]
	END

GO


IF EXISTS(SELECT TOP 1 * FROM sys.table_types tt
WHERE tt.name = 'DimensionKeys_Type')
	BEGIN
	
	PRINT N'Dropping [dbo].[DimensionKeys_Type]...';	
		DROP TYPE [dbo].[DimensionKeys_Type]
	END
GO

PRINT N'Creating [dbo].[DimensionKeys_Type]...';
CREATE TYPE [dbo].[DimensionKeys_Type] AS TABLE(
	[DimensionKeyId] [bigint] NOT NULL,
	[DimensionKey] [nvarchar](MAX) NOT NULL,
	PRIMARY KEY CLUSTERED 
(
	[DimensionKeyId] ASC
)WITH (IGNORE_DUP_KEY = OFF)
)
GO

PRINT N'Creating [dbo].[Ensure_DimensionKeys]...';

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
Create PROCEDURE [dbo].[Ensure_DimensionKeys]
  @DimensionKeyId [bigint],
  @DimensionKey [nvarchar](max)
AS
BEGIN
SET NOCOUNT ON;
BEGIN TRY
MERGE [DimensionKeys] AS Target USING ( VALUES (@DimensionKeyId, @DimensionKey)) AS Source (DimensionKeyId, DimensionKey)
 ON 
Target.[DimensionKeyId] = Source.[DimensionKeyId]
WHEN NOT MATCHED THEN
  INSERT ([DimensionKeyId], [DimensionKey]
)
VALUES (
Source.[DimensionKeyId], Source.[DimensionKey]);
END TRY
  BEGIN CATCH

    DECLARE @error_number INTEGER = ERROR_NUMBER();
    DECLARE @error_severity INTEGER = ERROR_SEVERITY();
    DECLARE @error_state INTEGER = ERROR_STATE();
    DECLARE @error_message NVARCHAR(4000) = ERROR_MESSAGE();
    DECLARE @error_procedure SYSNAME = ERROR_PROCEDURE();
    DECLARE @error_line INTEGER = ERROR_LINE();
IF( @@ERROR != 2627 )
    BEGIN

      RAISERROR( N'T-SQL ERROR %d, SEVERITY %d, STATE %d, PROCEDURE %s, LINE %d, MESSAGE: %s', @error_severity, 1, @error_number, @error_severity, @error_state, @error_procedure, @error_line, @error_message ) WITH NOWAIT;

    END
END CATCH;
END

GO
PRINT N'Creating [dbo].[Ensure_DimensionKeys_Tvp]...';

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [dbo].[Ensure_DimensionKeys_Tvp]
  @table [DimensionKeys_Type] READONLY
AS
BEGIN
  SET NOCOUNT ON;

  BEGIN TRY

    MERGE [DimensionKeys] AS Target USING @table Source
    ON 
      Target.[DimensionKeyId] = Source.[DimensionKeyId]
    WHEN NOT MATCHED THEN
      INSERT ([DimensionKeyId], [DimensionKey])
      VALUES (Source.[DimensionKeyId], Source.[DimensionKey]);

  END TRY
  BEGIN CATCH

    DECLARE @error_number INTEGER = ERROR_NUMBER();
    DECLARE @error_severity INTEGER = ERROR_SEVERITY();
    DECLARE @error_state INTEGER = ERROR_STATE();
    DECLARE @error_message NVARCHAR(4000) = ERROR_MESSAGE();
    DECLARE @error_procedure SYSNAME = ERROR_PROCEDURE();
    DECLARE @error_line INTEGER = ERROR_LINE();

    RAISERROR( N'T-SQL ERROR %d, SEVERITY %d, STATE %d, PROCEDURE %s, LINE %d, MESSAGE: %s', @error_severity, 1, @error_number, @error_severity, @error_state, @error_procedure, @error_line, @error_message ) WITH NOWAIT;
  END CATCH;
END
GO

PRINT N'Refreshing [dbo].[Ensure_DimensionKeys]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[Ensure_DimensionKeys]';


GO
PRINT N'Refreshing [dbo].[Ensure_DimensionKeys_Tvp]...';


GO
EXECUTE sp_refreshsqlmodule N'[dbo].[Ensure_DimensionKeys_Tvp]';


GO
PRINT N'Update complete.';


GO
